<?php

defined('_JEXEC') or die('Direct Access to ' . basename(__FILE__) . ' is not allowed.');

/**
 *  Multisafepay for Virtuemart 2
 *  Main payment class
 *
 *  @version 2.0.1
 *  @author Ruud Jonk
 *  @copyright Copyright (c) 2012 Multisafepay
 *
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See /administrator/components/com_virtuemart/COPYRIGHT.php for copyright notices and details.
 *
 * http://virtuemart.net
 */
 
if (!class_exists('vmPSPlugin'))
    require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');

require_once(dirname(__FILE__).'/multisafepay_api/MultiSafepay.combined.php');

class plgVMPaymentMultisafepay extends vmPSPlugin {

    // instance of class
    public static $_this = false;
    private $_multisafepay;
    public $_version = "2.0.5";

    function __construct(& $subject, $config) {
	//if (self::$_this)
	 //   return self::$_this;
	parent::__construct($subject, $config);

	$this->_loggable = true;
	$this->tableFields = array_keys($this->getTableSQLFields());

        /* stored to database based on settings */
	$varsToPush = array(
        'multisafepay_account_id'    	=> array('', 'char'),
		'payment_logos' 				=> array('', 'char'),
		'sandbox'    					=> array('', 'int'),
		'multisafepay_site_id'    		=> array('', 'char'),
	    'multisafepay_secure_code'    	=> array('', 'char'),
	    'multisafepay_paymentmethod' 	=> array('', 'char'),
	    'status_initialized'            => array('', 'char'),
	    'status_completed'            	=> array('', 'char'),
	    'status_uncleared'           	=> array('', 'char'),
	    'status_void'             		=> array('', 'char'),
	    'status_declined'         		=> array('', 'char'),
		'status_refunded'         		=> array('', 'char'),
		'status_expired'         		=> array('', 'char'),
		'status_canceled'         		=> array('', 'char'),
	    'payment_currency'          	=> array(0, 'int'),
	    'countries'                 	=> array(0, 'char'),
	    'min_amount'                	=> array(0, 'int'),
	    'max_amount'                	=> array(0, 'int'),
	    'cost_per_transaction'      	=> array(0, 'int'),
	    'cost_percent_total'        	=> array(0, 'int'),
	    'tax_id'                    	=> array(0, 'int')
	);

	$this->setConfigParameterable($this->_configTableFieldName, $varsToPush);
    }

    private function _getVersion(){
        return intval(str_replace(".", "", vmVersion::$RELEASE));
    }

    protected function getVmPluginCreateTableSQL() {
		return $this->createTableSQL('Payment Multisafepay Table');
    }

    function getTableSQLFields() 
	{
        $SQLfields = array(
	    'id' 							=> 'int(11) unsigned NOT NULL AUTO_INCREMENT',
	    'virtuemart_order_id' 			=> 'int(11) UNSIGNED DEFAULT NULL',
	    'order_number' 					=> 'char(32) DEFAULT NULL',
	    'virtuemart_paymentmethod_id' 	=> 'mediumint(1) UNSIGNED DEFAULT NULL',
	    'payment_name' 					=> 'char(255) NOT NULL DEFAULT \'\' ',
	    'payment_order_total' 			=> 'decimal(15,5) NOT NULL DEFAULT \'0.00000\' ',
	    'payment_currency' 				=> 'char(3) ',
	    'cost_per_transaction' 			=> ' decimal(10,2) DEFAULT NULL ',
	    'cost_percent_total' 			=> ' decimal(10,2) DEFAULT NULL ',
	    'tax_id' 						=> 'smallint(11) DEFAULT NULL',
        'multisafepay_order_id' 		=> 'int(11) UNSIGNED DEFAULT NULL',
        'multisafepay_transaction_id' 	=> 'char(32) DEFAULT NULL',
        'multisafepay_status' 			=> 'char(32) DEFAULT \'NEW\''
	);

	return $SQLfields;
    }

    private function _getLangISO(){
		$lang 	= &JFactory::getLanguage();
		$arr 	= explode("-",$lang->get('tag'));
		return strtoupper($arr[0]);
    }

    function plgVmConfirmedOrder($cart, $order) {

		if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_paymentmethod_id))) {
			return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($method->payment_element)) {
			return false;
		}
		
		$lang		 	= JFactory::getLanguage();
		$filename 		= 'com_virtuemart';
		$lang->load($filename, JPATH_ADMINISTRATOR);
		$vendorId 		= 0;

		if (!class_exists('VirtueMartModelOrders'))
			require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );
		
		$this->getPaymentCurrency($method);
		// END printing out HTML Form code (Payment Extra Info)
		$q 						= 'SELECT `currency_code_3` FROM `#__virtuemart_currencies` WHERE `virtuemart_currency_id`="' . $method->payment_currency . '" ';
		$db 					= &JFactory::getDBO();
		$db->setQuery($q);
		$currency_code_3 		= $db->loadResult();
		$paymentCurrency 		= CurrencyDisplay::getInstance($method->payment_currency);
		$totalInPaymentCurrency = round($paymentCurrency->convertCurrencyTo($method->payment_currency, $order['details']['BT']->order_total, false), 2);
		$cd 					= CurrencyDisplay::getInstance($cart->pricesCurrency);

		$address 				= ((isset($order['details']['ST'])) ? $order['details']['ST'] : $order['details']['BT']);

		$this->_virtuemart_paymentmethod_id = $order['details']['BT']->virtuemart_paymentmethod_id;
		
		$dbValues['order_number'] 					= $order['details']['BT']->order_number;
		$dbValues['payment_name'] 					= $this->renderPluginName($method);
		$dbValues['virtuemart_paymentmethod_id']	= $this->_virtuemart_paymentmethod_id;
		$dbValues['cost_per_transaction'] 			= $method->cost_per_transaction;
		$dbValues['cost_percent_total'] 			= $method->cost_percent_total;
		$dbValues['payment_currency'] 				= $currency_code_3 ;
		$dbValues['payment_order_total'] 			= $totalInPaymentCurrency;
		$dbValues['tax_id'] 						= $method->tax_id;
		$dbValues['multisafepay_order_id'] 			= $cart->virtuemart_order_id;
		$dbValues['multisafepay_status'] 			=  "NEW";
        $this->storePSPluginInternalData($dbValues);

        $amount 	= $totalInPaymentCurrency * 100;
        $returnURL 	= JROUTE::_(JURI::root() . 'index.php?option=com_virtuemart&view=multisafepayresponse&task=result&pm=' . $order['details']['BT']->virtuemart_paymentmethod_id);
		$Nurl 		= JROUTE::_(JURI::root() . 'index.php?option=com_virtuemart&view=multisafepayresponse&task=notify&mode=notify&type=initial&pm=' . $order['details']['BT']->virtuemart_paymentmethod_id);
		
		$items = '<ul>';
		foreach($order['items'] as $k){
			$items .= '<li>' . $k->product_quantity . ' x ' . $k->order_item_name . '</li>';
		}
		$items .= '</ul>';
	  
		$lang 								=& JFactory::getLanguage();

		$locale 							= $lang->getTag();
		$locale 							= str_replace('-', '_', $locale);
		$msp = new MultiSafepay();
		$msp->test                         	= $method->sandbox == 1 ? true : false;
		$msp->merchant['account_id']       	= $method->multisafepay_account_id;
		$msp->merchant['site_id']          	= $method->multisafepay_site_id;
		$msp->merchant['site_code']        	= $method->multisafepay_secure_code;
		$msp->merchant['notification_url'] 	= $Nurl;
		$msp->merchant['cancel_url']       	= JROUTE::_(JURI::root() . 'index.php?option=com_virtuemart&view=pluginresponse&task=pluginUserPaymentCancel&on=' . $order['details']['BT']->order_number . '&pm=' . $order['details']['BT']->virtuemart_paymentmethod_id);
		$msp->merchant['redirect_url'] 	   	= $returnURL;
		$msp->customer['locale']           	= $locale;
		$msp->customer['firstname']        	= $order['details']['BT']->first_name;
		$msp->customer['lastname']         	= $order['details']['BT']->last_name;
		$msp->customer['zipcode']          	= $order['details']['BT']->zip;
		$msp->customer['city']             	= $order['details']['BT']->city;
		$msp->customer['country']          	= ShopFunctions::getCountryByID($address->virtuemart_country_id, 'country_2_code');
		$msp->customer['phone']            	= $order['details']['BT']->phone_1;
		$msp->customer['email']            	= $order['details']['BT']->email;
		$msp->parseCustomerAddress($order['details']['BT']->address_1);	
		$msp->transaction['id']            	= $order['details']['BT']->order_number; // generally the shop's order ID is used here
		$msp->transaction['currency']      	= $currency_code_3;
		$msp->transaction['amount']        	= $amount;//$totalInPaymentCurrency * 100; // cents
		$msp->transaction['description']   	= 'Order #' . $msp->transaction['id'];
		$msp->transaction['items']         	= $items;
		$msp->plugin_name					= 'Virtuemart 2.X';
		$msp->version						= '2.0.6';
		
		$msp->plugin['shop']				= 'Virtuemart';
		$msp->plugin['shop_version']		= '2';
		$msp->plugin['plugin_version']		= '2.0.6';
		$msp->plugin['partner']				= '';
		$msp->plugin['shop_root_url']		= JURI::root();
		
		$url 								= $msp->startTransaction();
	  
		if ($msp->error)
		{
			$html = "Error " . $msp->error_code . ": " . $msp->error;
		}
		else
		{
			$html = '<p>'.JText::_('VMPAYMENT_MULTISAFEPAY_CLICK_ON_BUTTON').'<p>';
			$html.= '<form action="' . $url . '" method="post" name="vm_multisafepay_form" >';
			$html.= '<input type="image" name="submit" src="http://www.multisafepay.com/downloads/betaalbutton/connect/msp-connect-betaalmethoden-liggend-450px.gif" alt="'.JText::_('VMPAYMENT_MULTISAFEPAY_PAYNOW').'" />';
			$html.= '</form>';
			$html.= ' <script type="text/javascript">';
			$html.= ' document.vm_multisafepay_form.submit();';
			$html.= ' </script>';
		}

        if (!class_exists('VirtueMartModelOrders'))  require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );
        if (!class_exists('VirtueMartCart')) require(JPATH_VM_SITE . DS . 'helpers' . DS . 'cart.php');

      
       /* $modelOrder 	= new VirtueMartModelOrders();
        $orderitems 	= $modelOrder->getOrder($cart->virtuemart_order_id);
        
        //Ok so the functions have changed between Virtuemart 2.0.0 and 2.0.2 (great!)
        switch ($this->_getVersion()){
            case 200:
                $return = $this->processConfirmedOrderPaymentResponse(2, $cart, $order, $html); // keep cart, send order
                break;
            default:
                $return = $this->processConfirmedOrderPaymentResponse(2, $cart, $order, $html, $dbValues['payment_name'], $method->status_initialized);
        }
        return $return;
		
		*/
		$modelOrder = VmModel::getModel ('orders');

		$order['customer_notified'] = 1;
		$order['comments'] = '';
		$modelOrder->updateStatusForOneOrder ($order['details']['BT']->virtuemart_order_id, $order, TRUE);
		
		// $this->processConfirmedOrderPaymentResponse(2, $cart, $order, $html, $dbValues['payment_name'], $method->status_initialized);		
		
		$cart->_confirmDone = false;
		$cart->_dataValidated = false;
		$cart->setCartIntoSession();
		JRequest::setVar('html', $html);
		echo $html;
		die();

    }

    function plgVmgetPaymentCurrency($virtuemart_paymentmethod_id, &$paymentCurrencyId)
	{
		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
			return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($method->payment_element)) {
			return false;
		}
		$this->getPaymentCurrency($method);
		$paymentCurrencyId = $method->payment_currency;
    }


    /* Function to handle all responses */
    public function plgVmOnPaymentResponseReceived(  &$html) {
		if (!class_exists('VirtueMartModelOrders'))
			require_once( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );
       
		$virtuemart_paymentmethod_id = JRequest::getInt('pm', 0);
		$vendorId = 0;
		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
			return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($method->payment_element)) {
			return false;
		}
		
		$msp 								= new MultiSafepay();
		$order_number 						= $_GET['transactionid'];
		$modelOrder 						= new VirtueMartModelOrders();
		$order_id 							= $modelOrder->getOrderIdByOrderNumber($order_number);
		$order_object 						= $modelOrder->getOrder($order_id);
		
		$msp->test                         	= $method->sandbox == 1 ? true : false;
		$msp->merchant['account_id']       	= $method->multisafepay_account_id;
		$msp->merchant['site_id']          	= $method->multisafepay_site_id;
		$msp->merchant['site_code']        	= $method->multisafepay_secure_code;
		$msp->transaction['id']            	= $order_number; 
		$status 							= $msp->getStatus();	
        $details 							= $msp->details;

		switch ($status){
			case "initialized":
				JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_MULTISAFEPAY_MSG_INITIALIZED'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
            case "completed":
                JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_MULTISAFEPAY_MSG_COMPLETED'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
			case "uncleared":
                JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_UNCLEARED_MSG_UNCLEARED'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
            case "void":
                JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_MULTISAFEPAY_MSG_VOID'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
			case "declined":
                JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_MULTISAFEPAY_MSG_DECLINED'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
			case "refunded":
                JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_MULTISAFEPAY_MSG_REFUNDED'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
			case "expired":
                JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_MULTISAFEPAY_MSG_EXPIRED'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
			case "canceled":
                JRequest::setVar('multisafepay_msg', Jtext::_('VMPAYMENT_MULTISAFEPAY_MSG_CANCELED'));
                $html = $this->_getPaymentResponseHtml($details , $this->renderPluginName($method));
                break;
        }
		$order = array();
		
		switch ($status) {
			case "initialized": // waiting
				$order['order_status'] = $method->status_initialized;
				break;
			case "completed":   // payment complete
				$order['order_status'] = $method->status_completed;
				break;
			case "uncleared":   // waiting (credit cards or direct debit)
		  		$order['order_status'] = $method->status_uncleared;
				break;
			case "void":        // canceled
		  		$order['order_status'] = $method->status_void;
				break;
			case "declined":    // declined
		  		$order['order_status'] = $method->status_declined;
				break;
			case "refunded":    // refunded
		  		$order['order_status'] = $method->status_refunded;
				break;
			case "expired":     // expired
		  		$order['order_status'] = $method->status_expired;
				break;
			case "canceled":     // expired
		  		$order['order_status'] = $method->status_canceled;
				break;		
		}
		$payment 			= $this->getDataByOrderId($order_id);
		$current_status 	= $payment->multisafepay_status;
		
		$response_fields 	= array();

        $copyValue = array(
            "order_number",
            "virtuemart_paymentmethod_id",
            "payment_name",
            "payment_order_total",
            "payment_currency",
            "cost_per_transaction",
            "cost_percent_total",
            "tax_id",
            "cost_percent_total",
            "cost_percent_total",
            "cost_percent_total",
        );

        foreach ($copyValue as $value) {
            $response_fields[$value] = $payment->$value;
        }
		
		$response_fields['multisafepay_status']     = $status;

        switch ($this->_getVersion()){
			case 200:
				$this->storePSPluginInternalData($response_fields);
				break;
			default:
                $this->storePSPluginInternalData($response_fields, 'virtuemart_order_id', true);
        }
			
		if($order['order_status'] != $order_object['details']['BT']->order_status AND $order_object['details']['BT']->order_status != 'S' )//$current_status != $status  && 
		{
			$order['virtuemart_order_id'] = $order_id;
			$order['comments'] = '';
			if($order['order_status'] != $method->status_canceled)
			{
				$order['customer_notified'] = 1;//validate this one, can we trigger the notify customer for the initial pending status? 
			}
			else
			{
				$order['customer_notified'] = 0; 
			}
			$modelOrder->updateStatusForOneOrder($order_id, $order, true);
		}
		if($status != 'canceled')
		{
			$this->emptyCart(null);
		}
		return $html;
		//exit();
    }

    function plgVmOnUserPaymentCancel() {

	if (!class_exists('VirtueMartModelOrders'))
	    require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );

	$order_number = $_GET['transactionid'];
	$virtuemart_paymentmethod_id = JRequest::getInt('pm', '');
	if (empty($order_number) or empty($virtuemart_paymentmethod_id) or !$this->selectedThisByMethodId($virtuemart_paymentmethod_id)) {
	    return null;
	}
	if (!($virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number))) {
		return null;
	}
	if (!($paymentTable = $this->getDataByOrderId($virtuemart_order_id))) {
	    return null;
	}

	VmInfo(Jtext::_('VMPAYMENT_PAYPAL_PAYMENT_CANCELLED'));
	$session = JFactory::getSession();
	$return_context = $session->getId();
	/*if (strcmp($paymentTable->paypal_custom, $return_context) === 0) {
	    $this->handlePaymentUserCancel($virtuemart_order_id);
	}*/
	$this->handlePaymentUserCancel($virtuemart_order_id);
	return true;
    }


    function plgVmOnPaymentNotification() {
    }

    /**
     * Display stored payment data for an order
     * @see components/com_virtuemart/helpers/vmPSPlugin::plgVmOnShowOrderBEPayment()
     */
    function plgVmOnShowOrderBEPayment($virtuemart_order_id, $payment_method_id) {

		if (!$this->selectedThisByMethodId($payment_method_id)) {
			return null; // Another method was selected, do nothing
		}

		$db = JFactory::getDBO();
		$q = 'SELECT * FROM `' . $this->_tablename . '` '
			. 'WHERE `virtuemart_order_id` = ' . $virtuemart_order_id;
		$db->setQuery($q);

		if (!($paymentTable = $db->loadObject())) {
		   // JError::raiseWarning(500, $db->getErrorMsg());
			return '';
		}
		$this->getPaymentCurrency($paymentTable);
		$q = 'SELECT `currency_code_3` FROM `#__virtuemart_currencies` WHERE `virtuemart_currency_id`="' . $paymentTable->payment_currency . '" ';
		$db = &JFactory::getDBO();
		$db->setQuery($q);
		$currency_code_3 = $db->loadResult();
		$html = '<table class="adminlist">' . "\n";
		$html .=$this->getHtmlHeaderBE();
		$html .= $this->getHtmlRowBE('MULTISAFEPAY_PAYMENT_NAME', $paymentTable->payment_name);
		$html .= $this->getHtmlRowBE('MULTISAFEPAY_PAYMENT_TOTAL_CURRENCY', $paymentTable->payment_order_total.' '.$currency_code_3);

		$html .= '</table>' . "\n";
		return $html;

    }


	function _getPaymentResponseHtml($data, $payment_name) {
		$html = '<table cellspacing="0" cellpadding="0" class="multisafepay-table">' . "\n";
		$html .= $this->getHtmlRow('MULTISAFEPAY_PAYMENT_NAME', $payment_name);
		$html .= $this->getHtmlRow('MULTISAFEPAY_STATUS', $data['ewallet']['status']);
		$html .= $this->getHtmlRow('MULTISAFEPAY_PAYMENT_TRANSACTIONID', $data['transaction']['id']);
		$html .= '</table>' . "\n";

		return $html;
    }

    function getCosts(VirtueMartCart $cart, $method, $cart_prices) {
		if (preg_match('/%$/', $method->cost_percent_total)) {
			$cost_percent_total = substr($method->cost_percent_total, 0, -1);
		} else {
			$cost_percent_total = $method->cost_percent_total;
		}
		return ($method->cost_per_transaction + ($cart_prices['salesPrice'] * $cost_percent_total * 0.01));
    }

    /**
     * Check if the payment conditions are fulfilled for this payment method
     * @author: Valerie Isaksen
     *
     * @param $cart_prices: cart prices
     * @param $payment
     * @return true: if the conditions are fulfilled, false otherwise
     *
     */
    protected function checkConditions($cart, $method, $cart_prices) {
		$address = (($cart->ST == 0) ? $cart->BT : $cart->ST);

		$amount = $cart_prices['salesPrice'];
		$amount_cond = ($amount >= $method->min_amount AND $amount <= $method->max_amount
			OR
			($method->min_amount <= $amount AND ($method->max_amount == 0) ));

		$countries = array();
		if (!empty($method->countries)) {
			if (!is_array($method->countries)) {
			$countries[0] = $method->countries;
			} else {
			$countries = $method->countries;
			}
		}
		// probably did not gave his BT:ST address
		if (!is_array($address)) {
			$address = array();
			$address['virtuemart_country_id'] = 0;
		}

		if (!isset($address['virtuemart_country_id']))
			$address['virtuemart_country_id'] = 0;
		if (in_array($address['virtuemart_country_id'], $countries) || count($countries) == 0) {
			if ($amount_cond) {
			return true;
	    }
	}

	return false;
    }

    /**
     * We must reimplement this triggers for joomla 1.7
     */

    /**
     * Create the table for this plugin if it does not yet exist.
     * This functions checks if the called plugin is active one.
     * When yes it is calling the standard method to create the tables
     * @author Val�rie Isaksen
     *
     */
    function plgVmOnStoreInstallPaymentPluginTable($jplugin_id) {

	return $this->onStoreInstallPluginTable($jplugin_id);
    }

    /**
     * This event is fired after the payment method has been selected. It can be used to store
     * additional payment info in the cart.
     *
     * @author Max Milbers
     * @author Val�rie isaksen
     *
     * @param VirtueMartCart $cart: the actual cart
     * @return null if the payment was not selected, true if the data is valid, error message if the data is not vlaid
     *
     */
    public function plgVmOnSelectCheckPayment(VirtueMartCart $cart) {
	return $this->OnSelectCheck($cart);
    }

    /**
     * plgVmDisplayListFEPayment
     * This event is fired to display the pluginmethods in the cart (edit shipment/payment) for exampel
     *
     * @param object $cart Cart object
     * @param integer $selected ID of the method selected
     * @return boolean True on succes, false on failures, null when this plugin was not selected.
     * On errors, JError::raiseWarning (or JError::raiseError) must be used to set a message.
     *
     * @author Valerie Isaksen
     * @author Max Milbers
     */
    public function plgVmDisplayListFEPayment(VirtueMartCart $cart, $selected = 0, &$htmlIn) {
	return $this->displayListFE($cart, $selected, $htmlIn);
    }

    /*
     * plgVmonSelectedCalculatePricePayment
     * Calculate the price (value, tax_id) of the selected method
     * It is called by the calculator
     * This function does NOT to be reimplemented. If not reimplemented, then the default values from this function are taken.
     * @author Valerie Isaksen
     * @cart: VirtueMartCart the current cart
     * @cart_prices: array the new cart prices
     * @return null if the method was not selected, false if the shiiping rate is not valid any more, true otherwise
     *
     *
     */

    public function plgVmonSelectedCalculatePricePayment(VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {
	return $this->onSelectedCalculatePrice($cart, $cart_prices, $cart_prices_name);
    }

    /**
     * plgVmOnCheckAutomaticSelectedPayment
     * Checks how many plugins are available. If only one, the user will not have the choice. Enter edit_xxx page
     * The plugin must check first if it is the correct type
     * @author Valerie Isaksen
     * @param VirtueMartCart cart: the cart object
     * @return null if no plugin was found, 0 if more then one plugin was found,  virtuemart_xxx_id if only one plugin is found
     *
     */
    function plgVmOnCheckAutomaticSelectedPayment(VirtueMartCart $cart, array $cart_prices = array()) {
	return $this->onCheckAutomaticSelected($cart, $cart_prices);
    }

    /**
     * This method is fired when showing the order details in the frontend.
     * It displays the method-specific data.
     *
     * @param integer $order_id The order ID
     * @return mixed Null for methods that aren't active, text (HTML) otherwise
     * @author Max Milbers
     * @author Valerie Isaksen
     */
    public function plgVmOnShowOrderFEPayment($virtuemart_order_id, $virtuemart_paymentmethod_id, &$payment_name) {
	  $this->onShowOrderFE($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
    }

    /**
     * This event is fired during the checkout process. It can be used to validate the
     * method data as entered by the user.
     *
     * @return boolean True when the data was valid, false otherwise. If the plugin is not activated, it should return null.
     * @author Max Milbers

      public function plgVmOnCheckoutCheckDataPayment($psType, VirtueMartCart $cart) {
      return null;
      }
     */

    /**
     * This method is fired when showing when priting an Order
     * It displays the the payment method-specific data.
     *
     * @param integer $_virtuemart_order_id The order ID
     * @param integer $method_id  method used for this order
     * @return mixed Null when for payment methods that were not selected, text (HTML) otherwise
     * @author Valerie Isaksen
     */
    function plgVmonShowOrderPrintPayment($order_number, $method_id) {
	return $this->onShowOrderPrint($order_number, $method_id);
    }

    /**
     * Save updated order data to the method specific table
     *
     * @param array $_formData Form data
     * @return mixed, True on success, false on failures (the rest of the save-process will be
     * skipped!), or null when this method is not actived.
     * @author Oscar van Eijk

      public function plgVmOnUpdateOrderPayment(  $_formData) {
      return null;
      }
     */
    /**
     * Save updated orderline data to the method specific table
     *
     * @param array $_formData Form data
     * @return mixed, True on success, false on failures (the rest of the save-process will be
     * skipped!), or null when this method is not actived.
     * @author Oscar van Eijk

      public function plgVmOnUpdateOrderLine(  $_formData) {
      return null;
      }
     */
    /**
     * plgVmOnEditOrderLineBE
     * This method is fired when editing the order line details in the backend.
     * It can be used to add line specific package codes
     *
     * @param integer $_orderId The order ID
     * @param integer $_lineId
     * @return mixed Null for method that aren't active, text (HTML) otherwise
     * @author Oscar van Eijk

      public function plgVmOnEditOrderLineBE(  $_orderId, $_lineId) {
      return null;
      }
     */

    /**
     * This method is fired when showing the order details in the frontend, for every orderline.
     * It can be used to display line specific package codes, e.g. with a link to external tracking and
     * tracing systems
     *
     * @param integer $_orderId The order ID
     * @param integer $_lineId
     * @return mixed Null for method that aren't active, text (HTML) otherwise
     * @author Oscar van Eijk

      public function plgVmOnShowOrderLineFE(  $_orderId, $_lineId) {
      return null;
      }
     */
    function plgVmDeclarePluginParamsPayment($name, $id, &$data) {
	return $this->declarePluginParams('payment', $name, $id, $data);
    }

    function plgVmSetOnTablePluginParamsPayment($name, $id, &$table) {
	return $this->setOnTablePluginParams($name, $id, $table);
    }

}

// No closing tag
